<?php

namespace App\Entity;

use App\Repository\RecipeRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

/**
 * A crafting recipe
 */
#[ORM\Entity(repositoryClass: RecipeRepository::class)]
class Recipe {
    use TBaseEntity;
    use TNamedEntity;

    /**
     * @var Machine|null The crafting machine
     */
    #[ORM\ManyToOne(inversedBy: 'recipes')]
    #[ORM\JoinColumn(nullable: false)]
    private ?Machine $machine = null;
    /**
     * @var float|null The crafting time, in seconds
     */
    #[ORM\Column]
    private ?float $craftingTime = null;

    /**
     * @var Collection<int, InputRecipeMaterial> The input materials
     */
    #[ORM\OneToMany(targetEntity: InputRecipeMaterial::class, mappedBy: 'recipe', orphanRemoval: true)]
    private Collection $inputMaterials;
    /**
     * @var Collection<int, OutputRecipeMaterial> The output materials
     */
    #[ORM\OneToMany(targetEntity: OutputRecipeMaterial::class, mappedBy: 'recipe', orphanRemoval: true)]
    private Collection $outputMaterials;

    /**
     * Initialization
     */
    public function __construct () {
        $this->inputMaterials = new ArrayCollection();
        $this->outputMaterials = new ArrayCollection();
    }

    /**
     * The crafting machine
     *
     * @return Machine|null The crafting machine
     */
    public function getMachine (): ?Machine {
        return $this->machine;
    }
    /**
     * Set the crafting machine
     *
     * @param Machine|null $machine The crafting machine
     *
     * @return $this
     */
    public function setMachine (?Machine $machine): static {
        $this->machine = $machine;

        return $this;
    }

    /**
     * The crafting time, in seconds
     *
     * @return float|null The crafting time, in seconds
     */
    public function getCraftingTime (): ?float {
        return $this->craftingTime;
    }
    /**
     * Set the crafting time, in seconds
     *
     * @param float $craftingTime The crafting time, in seconds
     *
     * @return $this
     */
    public function setCraftingTime (float $craftingTime): static {
        $this->craftingTime = $craftingTime;

        return $this;
    }

    /**
     * The input materials
     *
     * @return Collection<int, InputRecipeMaterial> The input materials
     */
    public function getInputMaterials (): Collection {
        return $this->inputMaterials;
    }
    /**
     * Add an input material
     *
     * @param InputRecipeMaterial $inputMaterial The input material
     *
     * @return $this
     */
    public function addInputMaterial (InputRecipeMaterial $inputMaterial): static {
        if (!$this->inputMaterials->contains($inputMaterial)) {
            $this->inputMaterials->add($inputMaterial);
            $inputMaterial->setRecipe($this);
        }

        return $this;
    }
    /**
     * Remove an input material
     *
     * @param InputRecipeMaterial $inputMaterial The input material
     *
     * @return $this
     */
    public function removeInputMaterial (InputRecipeMaterial $inputMaterial): static {
        if ($this->inputMaterials->removeElement($inputMaterial)) {
            // set the owning side to null (unless already changed)
            if ($inputMaterial->getRecipe() === $this) {
                $inputMaterial->setRecipe(null);
            }
        }

        return $this;
    }

    /**
     * The output materials
     *
     * @return Collection<int, OutputRecipeMaterial> The output materials
     */
    public function getOutputMaterials(): Collection
    {
        return $this->outputMaterials;
    }
    /**
     * Add an output material
     *
     * @param OutputRecipeMaterial $outputMaterial The output material
     *
     * @return $this
     */
    public function addOutputMaterial(OutputRecipeMaterial $outputMaterial): static
    {
        if (!$this->outputMaterials->contains($outputMaterial)) {
            $this->outputMaterials->add($outputMaterial);
            $outputMaterial->setRecipe($this);
        }

        return $this;
    }
    /**
     * Remove an output material
     *
     * @param OutputRecipeMaterial $outputMaterial The output material
     *
     * @return $this
     */
    public function removeOutputMaterial(OutputRecipeMaterial $outputMaterial): static
    {
        if ($this->outputMaterials->removeElement($outputMaterial)) {
            // set the owning side to null (unless already changed)
            if ($outputMaterial->getRecipe() === $this) {
                $outputMaterial->setRecipe(null);
            }
        }

        return $this;
    }
}
